// Illustrator Script to Move All Items to "Print Data" Layer and Delete Other Layers
function moveAllItemsToPrintDataLayer() {
    // Get the active document
    var doc = app.activeDocument;

    // Check if the "Print Data" layer exists
    var printDataLayer;
    try {
        printDataLayer = doc.layers.getByName("Print Data");
    } catch (e) {
        // If the layer doesn't exist, create it
        printDataLayer = doc.layers.add();
        printDataLayer.name = "Print Data";
    }

    // Loop through all layers in the document
    for (var i = doc.layers.length - 1; i >= 0; i--) {
        var layer = doc.layers[i];

        // Skip the "Print Data" layer itself to avoid recursion
        if (layer.name === "Print Data") {
            continue;
        }

        // Move all items in the current layer to the "Print Data" layer
        for (var j = layer.pageItems.length - 1; j >= 0; j--) {
            try {
                var item = layer.pageItems[j];
                item.move(printDataLayer, ElementPlacement.PLACEATEND);
            } catch (e) {
                $.writeln("Could not move item: " + e.message);
            }
        }

        // Delete the current layer after moving its items
        try {
            layer.remove();
        } catch (e) {
            $.writeln("Could not delete layer: " + e.message);
        }
    }

    // No alert is shown, operation completes silently
}

// Run the script
moveAllItemsToPrintDataLayer();

// Existing Script: Adjust Artboard and Create Circles
function adjustArtboardAndCreateCircles() {
    var doc = app.activeDocument;
    var artboard = doc.artboards[doc.artboards.getActiveArtboardIndex()];
    var artboardRect = artboard.artboardRect;

    var leftIncrease = 0.6 * 72;
    var rightIncrease = 0.6 * 72;
    var topIncrease = 1 * 72;
    var bottomIncrease = 1 * 72;

    var newArtboardRect = [
        artboardRect[0] - leftIncrease,
        artboardRect[1] + topIncrease,
        artboardRect[2] + rightIncrease,
        artboardRect[3] - bottomIncrease
    ];
    artboard.artboardRect = newArtboardRect;

    var regmarkLayer = doc.layers.add();
    regmarkLayer.name = "Regmark";

    var circleSize = 0.1968 * 72; // Convert inches to points
    var spacing = 12 * 72; // Spacing between circles (12 inches)
    var minDistance = 8 * 72; // Minimum distance between circles (8 inches)

    function createCircle(x, y) {
        var circle = doc.pathItems.ellipse(y, x, circleSize, circleSize);
        circle.filled = true;
        circle.stroked = false;

        var fillColor = new CMYKColor();
        fillColor.cyan = 100;
        fillColor.magenta = 100;
        fillColor.yellow = 100;
        fillColor.black = 100;
        circle.fillColor = fillColor;

        circle.moveToBeginning(regmarkLayer);
    }

    var leftX = newArtboardRect[0];
    var rightX = newArtboardRect[2] - circleSize;
    var topY = newArtboardRect[1];
    var bottomY = newArtboardRect[3];

    // Create top-left and top-right circles
    createCircle(leftX, topY - circleSize / 2);
    createCircle(rightX, topY - circleSize / 2);

    // Create side circles
    var currentY = topY - spacing;

    while (currentY - circleSize >= bottomY) {
        createCircle(leftX, currentY);
        createCircle(rightX, currentY);
        currentY -= spacing;
    }

    // Ensure bottom-left and bottom-right circles stay fully within the artboard
    createCircle(leftX, bottomY + circleSize);
    createCircle(rightX, bottomY + circleSize);
}

// Call the first script
adjustArtboardAndCreateCircles();

// Updated Script: Generate Postnet Barcodes with a 12th Checksum Digit
function generatePostnetBarcodes() {
    // Ensure all elements are placed in the "Print Data" layer
    var doc = app.activeDocument;
    var printDataLayer = getOrCreateLayer("Print Data");

    // Define barcode parameters
    var tallBarHeight = 9.53; // Tall bar height in mm
    var shortBarHeight = 3.81; // Short bar height in mm
    var barWidth = 1.52; // Fixed width of each bar in mm
    var totalWidth = 212; // Total barcode width in mm
    var digitCount = 10; // Number of random digits before the 11th digit

    // Postnet barcode mapping
    var postnetMapping = {
        '0': [1, 1, 0, 0, 0],
        '1': [0, 0, 0, 1, 1],
        '2': [0, 0, 1, 0, 1],
        '3': [0, 0, 1, 1, 0],
        '4': [0, 1, 0, 0, 1],
        '5': [0, 1, 0, 1, 0],
        '6': [0, 1, 1, 0, 0],
        '7': [1, 0, 0, 0, 1],
        '8': [1, 0, 0, 1, 0],
        '9': [1, 0, 1, 0, 0]
    };

    // Generate 10 random digits (shared for both barcodes)
    var baseDigits = '';
    for (var i = 0; i < digitCount; i++) {
        baseDigits += Math.floor(Math.random() * 10).toString();
    }

    // For barcodef, add '0' as the 11th digit
    var barcodefBase = baseDigits + '0';

    // For barcodeb, add '5' as the 11th digit
    var barcodebBase = baseDigits + '5';

    // Calculate the checksum (12th digit) for barcodef
    var barcodefChecksum = calculateChecksum(barcodefBase);

    // Calculate the checksum (12th digit) for barcodeb
    var barcodebChecksum = calculateChecksum(barcodebBase);

    // Combine the base and checksum to form complete barcodes
    var barcodef = barcodefBase + barcodefChecksum.toString();
    var barcodeb = barcodebBase + barcodebChecksum.toString();

    // Generate barcodes
    generateBarcode(barcodef, "barcodef", "bottom");
    generateBarcode(barcodeb, "barcodeb", "top");

    // Add regmarks
    addRegmarkF();
    addRegmarkB();

    function calculateChecksum(digits) {
        var sum = 0;
        for (var i = 0; i < digits.length; i++) {
            sum += parseInt(digits[i], 10);
        }
        var remainder = sum % 10;
        return remainder === 0 ? 0 : 10 - remainder;
    }

    function generateBarcode(barcodeDigits, groupName, position) {
        var totalBars = (barcodeDigits.length * 5) + 2;
        var totalBarWidth = barWidth * totalBars;
        var totalSpacing = totalWidth - totalBarWidth;
        var spacingBetweenBars = totalSpacing / (totalBars - 1);

        var artboard = doc.artboards[doc.artboards.getActiveArtboardIndex()];
        var artboardBounds = artboard.artboardRect;

        var mmToPoints = 2.83465;
        var inchToPoints = 72;

        var cmykColor = new CMYKColor();
        cmykColor.cyan = 100;
        cmykColor.magenta = 100;
        cmykColor.yellow = 100;
        cmykColor.black = 100;

        var barcodeStartX, barcodeStartY, textStartX, textStartY;

        if (groupName === "barcodef") {
            var regmarkHeight = 3 * mmToPoints;
            var regmarkWidth = (artboardBounds[2] - artboardBounds[0]) - inchToPoints;
            var regmarkX = artboardBounds[0] + inchToPoints / 2;
            var regmarkY = artboardBounds[3];

            barcodeStartY = regmarkY + regmarkHeight;
            barcodeStartX = regmarkX + regmarkWidth - totalWidth * mmToPoints;

            var artboardCenterX = (artboardBounds[2] + artboardBounds[0]) / 2;
            textStartX = artboardCenterX;
            textStartY = regmarkY + regmarkHeight + (7 * mmToPoints);

            var textGroup = printDataLayer.groupItems.add();
            var textItem = textGroup.textFrames.add();
            textItem.contents = barcodeDigits.slice(0, 11); // Only display first 11 digits
            textItem.left = textStartX - (textItem.width / 2);
            textItem.top = textStartY;
            textItem.textRange.characterAttributes.size = 15;
            textGroup.name = "textboxf";
        } else if (groupName === "barcodeb") {
            barcodeStartX = artboardBounds[0];
            barcodeStartY = artboardBounds[1] - (3 * mmToPoints);

            addTextboxB(barcodeDigits.slice(0, 11)); // Only display first 11 digits
        }

        var group = printDataLayer.groupItems.add();
        group.name = groupName;

        function drawBar(group, x, y, height, width) {
            var rect = group.pathItems.rectangle(y, x, width, -height);
            rect.filled = true;
            rect.fillColor = cmykColor;
            rect.stroked = false;
        }

        var startX = barcodeStartX;
        var startY = barcodeStartY;

        drawBar(group, startX, startY, tallBarHeight * mmToPoints, barWidth * mmToPoints);
        startX += (barWidth + spacingBetweenBars) * mmToPoints;

        for (var digitIndex = 0; digitIndex < barcodeDigits.length; digitIndex++) {
            var digit = barcodeDigits[digitIndex];
            var bars = postnetMapping[digit];

            for (var j = 0; j < bars.length; j++) {
                var height = (bars[j] ? tallBarHeight : shortBarHeight) * mmToPoints;
                drawBar(group, startX, startY, height, barWidth * mmToPoints);
                startX += (barWidth + spacingBetweenBars) * mmToPoints;
            }
        }

        drawBar(group, startX, startY, tallBarHeight * mmToPoints, barWidth * mmToPoints);

        if (groupName === "barcodeb") {
            group.rotate(180, true, true, true, true, Transformation.BOTTOMLEFT);
            group.left = artboardBounds[0] + (0.5 * inchToPoints);
        }
    }

    function addTextboxB(barcodeValue) {
        var artboard = doc.artboards[doc.artboards.getActiveArtboardIndex()];
        var artboardBounds = artboard.artboardRect;

        var inchToPoints = 72;

        var textX = (artboardBounds[0] + artboardBounds[2]) / 2;
        var textY = artboardBounds[1] - (0.25 * inchToPoints);

        var textGroup = printDataLayer.groupItems.add();
        var textItem = textGroup.textFrames.add();
        textItem.contents = barcodeValue.slice(0, 11); // Only display first 11 digits
        textItem.textRange.characterAttributes.size = 15;
        textItem.left = textX - (textItem.width / 2);
        textItem.top = textY;

        textItem.rotate(180);
        textGroup.name = "textboxb";
    }

    function addRegmarkF() {
        var artboard = doc.artboards[doc.artboards.getActiveArtboardIndex()];
        var artboardBounds = artboard.artboardRect;

        var mmToPoints = 2.83465;
        var inchToPoints = 72;

        var cmykColor = new CMYKColor();
        cmykColor.cyan = 100;
        cmykColor.magenta = 100;
        cmykColor.yellow = 100;
        cmykColor.black = 100;

        var regmarkHeight = 3 * mmToPoints;
        var regmarkWidth = (artboardBounds[2] - artboardBounds[0]) - inchToPoints;
        var regmarkX = artboardBounds[0] + inchToPoints / 2;
        var regmarkY = artboardBounds[3];

        var regmarkGroup = printDataLayer.groupItems.add();
        regmarkGroup.name = "regmarkf";

        var regmark = regmarkGroup.pathItems.rectangle(regmarkY, regmarkX, regmarkWidth, -regmarkHeight);
        regmark.filled = true;
        regmark.fillColor = cmykColor;
        regmark.stroked = false;
    }

    function addRegmarkB() {
        var artboard = doc.artboards[doc.artboards.getActiveArtboardIndex()];
        var artboardBounds = artboard.artboardRect;

        var mmToPoints = 2.83465;
        var inchToPoints = 72;

        var cmykColor = new CMYKColor();
        cmykColor.cyan = 100;
        cmykColor.magenta = 100;
        cmykColor.yellow = 100;
        cmykColor.black = 100;

        var regmarkHeight = 3 * mmToPoints;
        var regmarkWidth = (artboardBounds[2] - artboardBounds[0]) - inchToPoints;
        var regmarkX = artboardBounds[0] + inchToPoints / 2;
        var regmarkY = artboardBounds[1] - regmarkHeight;

        var regmarkGroup = printDataLayer.groupItems.add();
        regmarkGroup.name = "regmarkb";

        var regmark = regmarkGroup.pathItems.rectangle(regmarkY, regmarkX, regmarkWidth, -regmarkHeight);
        regmark.filled = true;
        regmark.fillColor = cmykColor;
        regmark.stroked = false;
    }

    function getOrCreateLayer(layerName) {
        for (var i = 0; i < doc.layers.length; i++) {
            if (doc.layers[i].name === layerName) {
                return doc.layers[i];
            }
        }
        var newLayer = doc.layers.add();
        newLayer.name = layerName;
        return newLayer;
    }
}

// Call the second script
generatePostnetBarcodes();

// Illustrator Script to Move Specific Swatches to Kiss-cut and Thru-cut Layers
function moveSwatchesToLayers() {
    // Get the active document
    var doc = app.activeDocument;

    // Swatch names to match (case-insensitive)
    var kissCutSwatchName = "cutcontour";
    var thruCutSwatchNames = ["pounce", "perf", "thru"];

    // Create or find the Kiss-cut layer
    var kissCutLayer;
    try {
        kissCutLayer = doc.layers.getByName("Kiss-cut");
    } catch (e) {
        kissCutLayer = doc.layers.add();
        kissCutLayer.name = "Kiss-cut";
    }

    // Create or find the Thru-cut layer
    var thruCutLayer;
    try {
        thruCutLayer = doc.layers.getByName("Thru-cut");
    } catch (e) {
        thruCutLayer = doc.layers.add();
        thruCutLayer.name = "Thru-cut";
    }

    // Function to check if a swatch matches any target names (case-insensitive)
    function isMatchingSwatch(color, targetNames) {
        if (color.typename === "SpotColor") {
            var swatchName = color.spot.name.toLowerCase();
            for (var i = 0; i < targetNames.length; i++) {
                if (swatchName === targetNames[i]) {
                    return true;
                }
            }
        }
        return false;
    }

    // Iterate over all page items
    var items = doc.pageItems;
    for (var i = 0; i < items.length; i++) {
        var item = items[i];

        // Check for swatches in fills and strokes
        var hasKissCutSwatch = false;
        var hasThruCutSwatch = false;

        // Check fill
        if (item.filled) {
            var fillColor = item.fillColor;
            if (fillColor.typename === "SpotColor") {
                if (fillColor.spot.name.toLowerCase() === kissCutSwatchName) {
                    hasKissCutSwatch = true;
                } else if (isMatchingSwatch(fillColor, thruCutSwatchNames)) {
                    hasThruCutSwatch = true;
                }
            }
        }

        // Check stroke
        if (item.stroked) {
            var strokeColor = item.strokeColor;
            if (strokeColor.typename === "SpotColor") {
                if (strokeColor.spot.name.toLowerCase() === kissCutSwatchName) {
                    hasKissCutSwatch = true;
                } else if (isMatchingSwatch(strokeColor, thruCutSwatchNames)) {
                    hasThruCutSwatch = true;
                }
            }
        }

        // Move items to the appropriate layer
        if (hasKissCutSwatch) {
            item.move(kissCutLayer, ElementPlacement.PLACEATEND);
        } else if (hasThruCutSwatch) {
            item.move(thruCutLayer, ElementPlacement.PLACEATEND);
        }
    }

    alert("All matching items have been moved to their respective layers.");
}

// Call the new script
moveSwatchesToLayers();
